/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <mysql.h>
#include <NdbApi.hpp>

#include "config.h"

extern "C" int load_defaults(const char *conf_file, const char **groups,int *argc, char ***argv);

#define BAILOUT(obj, msg) do { fprintf(stderr, "%s: %s (%d) in %s:%d\n", msg, obj->getNdbError().message, obj->getNdbError().code, __FILE__, __LINE__); exit(3); } while (0);

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
	Ndb_cluster_connection *conn;
	char constr[1024];

	conn = example_init(&argc, &argv);

  Ndb* myNdb = new Ndb( conn,
			"TEST_DB_1" );  // Object representing the database
  if (myNdb->init() == -1) { 
    BAILOUT(myNdb, "can't init ndb object");
    exit(-1);
  }

  const NdbDictionary::Dictionary* myDict= myNdb->getDictionary();
  const NdbDictionary::Table *myTable= myDict->getTable("MYTABLENAME");
  
  if (myTable == NULL)
    BAILOUT(myDict, "can't lookup table");

  /**************************************************************************
   * Fill table with tuples, using auto_increment IDs                       *
   **************************************************************************/

  {
    for (int i = 0; i < 100; i++) {
      NdbTransaction *myTransaction= myNdb->startTransaction();
      if (myTransaction == NULL) BAILOUT(myNdb, "can't create transaction");
      
      NdbOperation *myOperation= myTransaction->getNdbOperation(myTable);
      if (myOperation == NULL) BAILOUT(myTransaction, "can't get operation");
      
      myOperation->insertTuple();
      

      if (myNdb->initAutoIncrement() == -1)
	BAILOUT(myNdb, "can't init autoinc");
     
      Uint64 auto_id = 0; 
      if (myNdb->getAutoIncrementValue(myTable,auto_id,0) == -1)
	BAILOUT(myNdb, "can't get autoinc");
        
      if (myOperation->equal("ATTR1",(int)auto_id)==-1)
        BAILOUT(myTransaction, "error on equal()");

      if (myOperation->setValue("ATTR2", i)==-1)
        BAILOUT(myTransaction, "error on setValue()");
      
      if (myTransaction->execute( NdbTransaction::Commit ) == -1)
	BAILOUT(myTransaction, "error on execute");
      
      myNdb->closeTransaction(myTransaction);
    }
    
  }


	example_end(conn);

	printf("OK!\n");

	return EXIT_SUCCESS;
}
